<?php

namespace Drupal\custom_admin_menu\Service;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Session\AccountProxyInterface;

/**
 * Provides tools to hide/display menu items
 */
class CustomAdminMenuMenuItemDisplayManager {

  /**
   * Nom du service.
   *
   * @const string
   */
  const SERVICE_NAME = 'custom_admin_menu.menu_item_display_manager';

  /**
   * Current User.
   *
   * @var AccountProxyInterface
   */
  protected $currentUser;

  /**
   * @param \Drupal\Core\Session\AccountProxyInterface $currentUser
   */
  public function __construct(AccountProxyInterface $currentUser) {
    $this->currentUser = $currentUser;
  }

  /**
   * Retourne le singleton.
   *
   * @return static
   *   Le singleton.
   */
  public static function me() {
    return \Drupal::service(static::SERVICE_NAME);
  }

  /**
   * {@inheritdoc}
   */
  public function initMenuItemsDisplayState(&$variables) {
    if (isset($variables['items'])) {
      foreach ($variables['items'] as $pluginId => $item) {
        if( $this->itemIsDisallowed($item) ){
          unset($variables['items'][$pluginId]);
        }
      }
    }
  }

  /**
   * Return true if item is explicitly disallowed.
   *
   * @param $pluginDefinition
   *   The item plugin definition.
   * @param \Drupal\Core\Session\AccountProxyInterface $user
   *   The user.
   *
   * @return bool
   *   The state.
   */
  /**
   * @param array $item
   *   Menu item data.
   * @param \Drupal\Core\Session\AccountProxyInterface|null $user
   *   THe user.
   *
   * @return bool
   *   The disallowed status.
   */
  protected function itemIsDisallowed($item, AccountProxyInterface $user = NULL) {
    $user = $user ?: $this->currentUser;

    if (isset($item['original_link'])) {
      $pluginDefinition = $item['original_link']->getPluginDefinition();
      if (!empty($pluginDefinition)) {
        $isDisallowed = FALSE;
        // Check disallowed roles.
        $disallowedRoles = NestedArray::getValue($pluginDefinition, [
          'metadata',
          'disallowed_roles'
        ]);
        if (!empty($disallowedRoles)) {
          $isDisallowed = count(array_intersect($user->getRoles(), $disallowedRoles)) > 0;
        }

        return $isDisallowed;
      }
    }

    return FALSE;
  }

}
